<?php
/*
 * Copyright (C) 2000-2002 ProgWeb
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.  Or, visit http://www.gnu.org/copyleft/lgpl.html
 */



/*
***************************************************************************************************
	Send some mails

											Author : Nicolas VIVIEN
											WebMaster : http://www.progweb.com
											Copyright  2002 Nicolas VIVIEN
***************************************************************************************************


		This class allowes to send mail with all features of mail clients.
		This class implements the function mail() in PHP



@version	3.0
	- All class has been rewrited
	- Method To, Cc, Bcc... can be called severals times
	- Documents inlne operate
	- Mail in HTML and in Text
	- All charset are managed by charset method 7bit, 8bit, quoted-printable
	- Format method is removed

@version	2.5
	- some updates to manage errors

@version	2.4
	- added getRapport method : method getRapport return a table with the errors
	- added RunTest method : this method allow testing configuration of script

@version	2.3
	- added a possibility to indicate the list of mails (array or string)
	- added disposition parameter inline and attachment

@version	2.2
	- added error traitement : method Get/Send return false or true if mail is correct

@version	2.1
	- added Format( "text"/"html" ) method, this allow send a mail in HTML
	- added a method to detect the mimes type of document (use the file gives wiith Apache)

@version	1.3
	- added ReplyTo( $address ) method
	- added Receipt() method - to add a mail receipt
	- added optionnal charset parameter to Body() method. this should fix charset problem on some mail clients



@sample

	include("libmail.inc");


	// Create the mail
	$m= new Mail();

	// Test installation of lib
	$m->RunTest();

	// Check mail or not (by default mails is checked)
	$m->CheckMail(false);

	// Set From
	$m->From( "from@somewhere.fr" );

	// Set To (it can be an array or a string)
	// ["mail1", "mail2"] or "mail1, mail2"
	$m->To( "destination@somewhere.fr" );

	// Set Subject
	$m->Subject( "the subject of the mail" );

	// Set Body (by default body is text)
	$message= "Hello world!\nthis is a test of the Mail class\nplease ignore\nThanks.";
	$m->Body( $message, ['text', ''html', text&html', 'html&text']);

	// Set Cc (it can be an array or a string)
	// ["mail1", "mail2"] or "mail1, mail2"
	$m->Cc( "someone@somewhere.fr");

	// Set Bcc (it can be an array or a string)
	// ["mail1", "mail2"] or "mail1, mail2"
	$m->Bcc( "someoneelse@somewhere.fr");

	// Set Reply To
	$m->ReplyTo( "someone@somewhere.fr" );

	// Set Organization
	$m->Organization( "name" );

	// Set Priority
	// 1 (Highest); 2 (High); 3 (Normal); 4 (Low); 5 (Lowest)
	$m->Priority(4);

	// Attach a file (it can be an array or a string)
	// ["file1", "file2"] or "file"
	// Set disposition with optional parameter (inline or attachment)
	// ["dispo1", "disp2"] or "dispo"
	$m->Attach( "toto.gif", "attachment" );

	// Get a reception (by default not)
	$m->Receipt();

	// Send the mail
	$m->Send();

	// Get the source mail
	$m->Get();

	// Get rapport of errors
	$rapport = $m->Rapport();
*/




class Mail {
	var $aTo = array();
	var $aCc = array();
	var $aBcc = array();
	var $aAttach = array();
	var $aDisposition = array();
	var $aFileType = array();
	var $aCid = array();
	var $aPriorities = array(1=>'1 (Highest)', '2 (High)', '3 (Normal)', '4 (Low)', '5 (Lowest)');
	var $aHeaders = array();
	var $aAllErrors = array();
	var $aBoundary = array();

	var $Headers;
	var $Body;
	var $BodyText;
	var $BodyHtml;
	var $Receipt = false;

	var $Charset = 'us-ascii';
	var $Encoding = "7bit";
	var $eof = "\n";

	var $error;
	var $checkmail;
	var $mime_file = 'mime.types';
	var $Version = 'libMailv3.0';
	var $compte;

	var $aErrors = array(
			101 => "From: L'expditeur du mail n'a pas t dfini",
			102 => "From: Syntaxe du mail incorrecte",

			201 => "To: Syntaxe du mail incorrecte",
			202 => "CC: Syntaxe du mail incorrecte",
			203 => "BCC: Syntaxe du mail incorrecte",
			204 => "BCC: Avertissement",
			205 => "Les destinataires du mail n'ont pas t dfinis",

			301 => "Document joint introuvable",

			401 => "L'objet du message n'a pas t dfini",

			501 => "Le corps du message est vide"
		);


	/*********************/
	/* Public functions */
	/*********************/

	function Mail() {
		$this->mime_file = ($GLOBALS['libdir']).($this->mime_file);

		$this->error = false;
		$this->CheckMail(true);
		$this->Charset('ISO-8859-15', 'quoted-printable');

		$this->aBoundary[0] = "=-" . md5(uniqid("ProgWeb"));
		$this->aBoundary[1] = "=-" . md5(uniqid("ProgWeb"));
		$this->aBoundary[2] = "=-" . md5(uniqid("ProgWeb"));
	}


	function CheckMail($bool) {
		if ($bool)
			$this->checkmail = true;
		else
			$this->checkmail = false;
	}


	function RunTest() {
		echo '<p>Script developp par ProgWeb : '.($this->Version).'<br>';
		echo 'Ce script est sous licence GNU/GPL';

		echo '<p><b>Test du fichier <i>mime.types</i> :</b><br>';
		echo 'Chemin indiqu&eacute; : <i>'.($this->mime_file).'</i><br>';
		echo 'Fichier trouv&eacute; : ';

		$res = (is_readable($this->mime_file)) ? 'oui' : 'non';
		echo '<font color="#00AA00"><b>'.($res).'</b></font>';


		echo '<p><b>Test de la pr&eacute;sence de la fonction <i>mail</i> :</b><br>';
		echo 'Fonction pr&eacute;sente : ';

		$res = (function_exists('mail')) ? 'oui' : 'non';
		echo '<font color="#00AA00"><b>'.($res).'</b></font>';

		if ($res == 'non') {
			echo '<p><b>Test de la pr&eacute;sence de la fonction <i>email</i> :</b><br>';
			echo 'Fonction pr&eacute;sente : ';

			$res = (function_exists('email')) ? 'oui' : 'non';
			echo '<font color="#00AA00"><b>'.($res).'</b></font>';
		}


		echo '<p><b>Test de la pr&eacute;sence de la fonction <i>imap_8bit</i> :</b><br>';
		echo 'Fonction pr&eacute;sente : ';

		$res = (function_exists('imap_8bit')) ? 'oui' : 'non';
		echo '<font color="#00AA00"><b>'.($res).'</b></font>';

		if ($res == 'non') {
			echo '<p><b>Avertissement :</b><br>';
			echo 'La fonction <i>imap_8bit</i> est pr&eacute;sente dans le module IMAP de PHP.<br>';
			echo "Elle permet d'envoyer des mails encoder sous la forme 'quoted-printable'";

			echo "<p>Sans cette fonction vous devez utiliser l'encodage '7bit' ou '8bit'<br>";
			echo "La librairie utilisera par d&eacute;faut un encodage en 8bit dans votre cas";
		}
	}


	function From($from) {
		$from = trim($from);

		$this->aHeaders['From'] = $from;
	}


	function To($to) {
		if (!is_array($to))
			$to = explode(",", $to);

		reset($to);
		array_walk($to, array(&$this, '_trim'));

		$this->aTo = array_merge($this->aTo, $to);
	}


	function Cc($cc) {
		if (!is_array($cc))
			$cc = explode(",", $cc);

		reset($cc);
		array_walk($cc, array(&$this, '_trim'));

		$this->aCc = array_merge($this->aCc, $cc);
	}


	function Bcc($bcc) {
		if (!is_array($bcc))
			$bcc = explode(",", $bcc);

		reset($bcc);
		array_walk($bcc, array(&$this, '_trim'));

		$this->aBcc = array_merge($this->aBcc, $bcc);
	}


	function ReplyTo($address) {
		$address = trim($address);
		$this->aHeaders["Reply-To"] = $address;
	}


	function Receipt() {
		$this->Receipt = true;
	}


	function Subject($subject) {
		$subject = ereg_replace("([\r\n\t])", " ", $subject);
		$this->aHeaders['Subject'] = $subject;
	}


	function Organization($name) {
		$name = trim($name);
		$this->aHeaders['Organization'] = $name;
	}


	function Priority($priority) {
		$this->aHeaders["X-Priority"] = $this->aPriorities[$priority];
	}


	function Body($body, $format='text') {
		switch ($format) {
			case 'text&html':
			case 'html&text':
				$this->BodyHtml = $body;
				$this->BodyText = $body;
				break;

			case 'html':
				$this->BodyHtml = $body;
				break;

			case 'text':
			default:
				$this->BodyText = $body;
		}
	}


	function Charset($charset, $encoding) {
		$this->Charset = $charset;

		if (function_exists('imap_8bit') && ($encoding == 'quoted-printable'))
			$this->Encoding = $encoding;
		else if ($encoding == 'quoted-printable')
			$this->Encoding = '8bit';
		else
			$this->Encoding = $encoding;
	}


	function Attach($filename, $disposition='attachment') {
		if (is_array($filename)) {
			$aAttach = $filename;

			if (is_array($disposition))
				$aDisposition = $disposition;
			else
				$aDisposition = array($disposition);
		}
		else {
			$aAttach = array($filename);
			$aDisposition = array($disposition);
		}

		$size = count($aAttach);
		$aDisposition = array_pad($aDisposition, $size, 'attachment');

		for($i=0; $i<$size; $i++) {
			$this->aAttach[] = $aAttach[$i];
			$this->aDisposition[] = $aDisposition[$i];
			$this->aFileType[] = $this->_getMimeType($aAttach[$i]);
		}
	}


	function Send() {
		$this->Headers = '';
		$this->Body = '';

		if ($this->_checkMail()) {
			$this->_buildMail();

			$to = implode(", ", $this->aTo);

			if (function_exists('mail'))
				$res = @mail($to, $this->aHeaders['Subject'], $this->Body, $this->Headers);
			else if (function_exists('email'))
				$res = @email($this->compte, $to, $this->aHeaders['Subject'], $this->Body, $this->Headers);
			else
				$res = false;

			return $res;
		}
		else
			return false;
	}


	function Get() {
		$this->Headers = '';
		$this->Body = '';

		$eof = $this->eof;

		$this->_buildMail();

		$to = implode(", ", $this->aTo);

		$src = "To: ".($to).($eof);
		$src .= "Subject: ".($this->aHeaders['Subject']).($eof);
		$src .= ($this->Headers).($eof);
		$src .= ($this->Body);

		return $src;
	}


	function Rapport() {
		$this->_checkMail();

		if (count($this->aAllErrors))
			return $this->aAllErrors;
		else
			return false;
	}



	/**********************/
	/* Private functions */
	/**********************/

	function _buildMail() {
		$i = 0;
		$eof = $this->eof;

		$boundary = $this->aBoundary;


		// Build headers of mail
		$this->_buildHeaders();


		$in_attachment = in_array('attachment', $this->aDisposition);
		$in_inline = in_array('inline', $this->aDisposition);

		if (empty($this->BodyHtml)) {
			if ($in_inline)
				$in_attachment = true;

			$in_inline = false;
		}

		if ($in_attachment) {
			$this->aHeaders['Content-Type'] = 'multipart/mixed; boundary="'.($boundary[$i]).'"';

			if ($in_inline) {
				$this->Body .= $eof;
				$this->Body .= '--'.($boundary[$i++]).($eof);
				$this->Body .= 'Content-Type: multipart/related; boundary="'.($boundary[$i]).'"'.($eof);
				$this->Body .= $eof;
			}
		}
		else if ($in_inline)
			$this->aHeaders['Content-Type'] = 'multipart/related; type="multipart/alternative"; boundary="'.($boundary[$i]).'"';


		if (isset($this->BodyHtml) && ($in_inline || $in_attachment)) {
			$this->Body .= $eof;
			$this->Body .= '--'.($boundary[$i++]).($eof);
			$this->Body .= 'Content-Type: multipart/alternative; boundary="'.($boundary[$i]).'"'.($eof);
			$this->Body .= $eof;
		}


		reset($this->aHeaders);

		while (list ($key, $value) = each($this->aHeaders)) {
			if ($key != "Subject")
				$this->Headers .= "$key: $value\n";
		}


		// Build body of mail (the text)
		$body = $this->_buildBody($boundary[$i]);

		if (isset($this->BodyHtml))
			$i--;


		// Finish body of mail
		$attach = '';

		if ($in_inline)
			$attach .= $this->_buildAttachment('inline', $boundary[$i--]);

		if ($in_attachment)
			$attach .= $this->_buildAttachment('attachment', $boundary[$i]);


		// Replace inline document
		$patterns = array();
		$replacements = array();

		reset($this->aCid);

		while (list ($key, $value) = each($this->aCid)) {
			$patterns[] = "#(<img src=['\"]{0,1})(".($value).")(['\"]{0,1}.*>)#si";
			$replacements[] = "\\1cid:".($key)."\\3";
		}

		$body = preg_replace($patterns, $replacements, $body);


		$this->Body .= ($body).($attach);
	}


	function _buildHeaders() {
		$eof = $this->eof;

		$XMailer = 'Php/'.($this->Version).' (ProgWeb.com)';


		if (count($this->aCc))
			$this->aHeaders['Cc'] = implode(", ", $this->aCc);

		if (count($this->aBcc))
			$this->aHeaders['Bcc'] = implode(", ", $this->aBcc);

		if ($this->Receipt) {
			if (isset($this->aHeaders['Reply-To']))
				$this->aHeaders['Disposition-Notification-To'] = $this->aHeaders['Reply-To'];
			else
				$this->aHeaders['Disposition-Notification-To'] = $this->aHeaders['From'];
		}

		if (isset($this->BodyHtml))
			$this->aHeaders['Content-Type'] = 'multipart/alternative; boundary="'.($this->aBoundary[0]).'"';
		else {
			$this->aHeaders['Content-Type'] = 'text/plain; charset="'.($this->Charset).'"';

			if (!count($this->aAttach))
				$this->aHeaders['Content-Transfer-Encoding'] = $this->Encoding;
		}

		$this->aHeaders['X-Mailer'] = $XMailer;
		$this->aHeaders['Date'] = date("r");
		$this->aHeaders['Message-Id'] = '<'.($this->_getRandCid()).'>';
		$this->aHeaders['Mime-Version'] = "1.0";
	}


	function _buildBody($boundary) {
		$eof = $this->eof;


		if (empty($this->BodyText))
			$this->BodyText = 'Vous devez possder un client Mail capable de lire du HTML';

		if (isset($this->BodyHtml)) {
			$out .= $eof;
			$out .= '--'.($boundary).($eof);
			$out .= 'Content-Type: text/plain; charset='.($this->Charset).'"'.($eof);
			$out .= 'Content-Transfer-Encoding: '.($this->Encoding).($eof);
			$out .= $eof;

			$buf = $this->BodyText;

			if ($this->Encoding == 'quoted-printable')
				$buf = imap_8bit($buf);

			$out .= ($buf).($eof);


			$out .= $eof;
			$out .= '--'.($boundary).($eof);
			$out .= 'Content-Type: text/html; charset="'.($this->Charset).'"'.($eof);
			$out .= $eof;
			$out .= ($this->BodyHtml).($eof);

			$out .= $eof;
			$out .= '--'.($boundary).'--'.($eof);

			$out .= $eof;
		}
		else {
			if (count($this->aAttach)) {
				$out .= $eof;
				$out .= '--'.($boundary).($eof);
				$out .= 'Content-Type: text/plain; charset="'.($this->Charset).'"'.($eof);
				$out .= 'Content-Transfer-Encoding: '.($this->Encoding).($eof);
				$out .= $eof;
			}

			$buf = $this->BodyText;

			if ($this->Encoding == 'quoted-printable')
				$buf = imap_8bit($buf);

			$out .= ($buf).($eof);
			$out .= $eof;
		}

		return $out;
	}


	function _buildAttachment($dispo, $boundary) {
		$out = '';
		$eof = $this->eof;

		for($i=0; $i<count($this->aAttach); $i++) {
			$filename = $this->aAttach[$i];
			$basename = basename($filename);

			$filetype = $this->aFileType[$i];
			$disposition = $this->aDisposition[$i];

			if (empty($this->BodyHtml))
				$disposition = 'attachment';

			if (($disposition == $dispo) && is_readable($filename)) {
				switch ($filetype) {
					case 'text/plain':
						$out .= '--'.($boundary).($eof);
						$out .= 'Content-Disposition: '.($disposition).'; filename="'.($basename).'"'.($eof);
						$out .= 'Content-Transfer-Encoding: '.($this->Encoding).($eof);
						$out .= 'Content-Type: '.($filetype).'; name="'.($basename).'"; charset="'.($this->Charset).'"'.($eof);
						$out .= $eof;

						$fp = fopen($filename, "r");
						$attachment = fread($fp, filesize($filename));

						if ($this->Encoding == 'quoted-printable')
							$attachment = imap_8bit($attachment);

						$out .= $attachment;
						fclose($fp);

						break;

					default:
						$out .= '--'.($boundary).($eof);

						if ($disposition == 'inline') {
							$cid = $this->_getRandCid();
							$out .= 'Content-ID: <'.($cid).'>'.($eof);

							$this->aCid[$cid] = $basename;
						}

						$out .= 'Content-Disposition: '.($disposition).'; filename="'.($basename).'"'.($eof);
						$out .= 'Content-Type: '.($filetype).'; name="'.($basename).'"'.($eof);
						$out .= 'Content-Transfer-Encoding: base64'.($eof);
						$out .= $eof;

						$fp = fopen($filename, "r");
						$attachment = fread($fp, filesize($filename));
						$out .= chunk_split(base64_encode($attachment));
						fclose($fp);
				}

				$out .= $eof;
			}
		}

		$out .= '--'.($boundary).'--'.($eof);
		$out .= $eof;

		return $out;
	}


	function _getMimeType($filename) {
		$len = strlen($filename);
		$pos = strrpos($filename, ".");

		if ($pos) {
			$pos++;
			$ext = substr($filename, $pos, $len-$pos);

			if ($fp = fopen($this->mime_file, "r")) {
				while (!feof($fp)) {
					$line = fgets($fp, 255);
					$line = trim($line);

					if (substr($line, 0, 1) == "#")
						continue;

					$line = str_replace(" ", "\t", $line);
					$tab = split("\t", $line);

					if (in_array($ext, $tab))
						return $tab[0];
				}

				fclose($fp);
			}
			else
				return "application/x-unknown-content-type";
		}

		return "application/x-unknown-content-type";
	}


	function _getRandCid() {
		return (1000000000+rand(0, 1000000000)).'.'.(1000+rand(0, 1000)).'@ProgWeb';
	}


	function _checkMail() {
		if ($this->checkmail == false)
			return true;

		$this->aAllErrors = array();


		/* Field From */
		// error 101
		if (empty($this->aHeaders['From']))
			$this->aAllErrors[] = array(101, $this->aErrors[101]);

		// error 102
		if (isset($this->aHeaders['From']) && !$this->_checkAddress($this->aHeaders['From']))
			$this->aAllErrors[] = array(102, $this->aErrors[102], $this->aHeaders['From']);
		/* End of field from */


		/* Field To, CC, Bcc */
		// error 201
		for ($i=0; $i<count($this->aTo); $i++)
			if (!$this->_checkAddress($this->aTo[$i]))
				$this->aAllErrors[] = array(201, $this->aErrors[201], $this->aTo[$i]);

		// error 202
		for ($i=0; $i<count($this->aCc); $i++)
			if (!$this->_checkAddress($this->aCc[$i]))
				$this->aAllErrors[] = array(202, $this->aErrors[202], $this->aCc[$i]);

		// error 203
		for ($i=0; $i<count($this->aBcc); $i++)
			if (!$this->_checkAddress($this->aBcc[$i]))
				$this->aAllErrors[] = array(203, $this->aErrors[203], $this->aBcc[$i]);

		// error 204
		if (!count($this->aTo) && !count($this->aCc) && count($this->aBcc))
			$this->aAllErrors[] = array(204, $this->aErrors[204]);

		// error 205
		if (!count($this->aTo) && !count($this->aCc) && !count($this->aBcc))
			$this->aAllErrors[] = array(205, $this->aErrors[205]);
		/* End of field To, CC, Bcc */


		/* Field Attachment */
		// error 301
		for ($i=0; $i<count($this->aAttach); $i++)
			if (!file_exists($this->aAttach[$i]))
				$this->aAllErrors[] = array(301, $this->aErrors[301], $this->aAttach[$i]);
		/* End of field attachment */


		/* Object of mail */
		// error 401
		if (empty($this->aHeaders['Subject']))
			$this->aAllErrors[] = array(401, $this->aErrors[401]);
		/* End of object of mail */


		/* Body of mail */
		if (empty($this->BodyText) && empty($this->BodyHtml))
			$this->aAllErrors[] = array(501, $this->aErrors[501]);
		/* End of object of mail */


		if (count($this->aAllErrors))
			return false;

		return true;
	}


	function _trim(&$value, $key) {
		$value = trim($value);
	}


	function _checkAddress($address) {
		if (ereg(".*<(.+)>", $address, $regs))
			$address = $regs[1];

 		if (ereg("^[^@  ]+@([a-zA-Z0-9\-]+\.)+([a-zA-Z0-9\-]{2,3}|info|nom|museulm)\$", $address))
 			return true;
 		else
 			return false;
	}
}
?>
